/*   Cataract - Static web photo gallery generator
 *   Copyright (C) 2008 Tomas Bzatek <tbzatek@users.sourceforge.net>
 *   
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License
 *   as published by the Free Software Foundation; either version 2
 *   of the License, or any later version.
 *   
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *   
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
#ifndef __ITEMS_H__ 
#define __ITEMS_H__ 


#include <glib.h>


typedef enum {
  GALLERY_TYPE_INDEX = 1 << 0,
  GALLERY_TYPE_ALBUM = 1 << 1
} TGalleryType;    

typedef struct {
  TGalleryType type;
  char *ID;
  char *title;
  char *desc;
  GPtrArray *items;
  char *base_dir;
  void *parent_index;    //  pointer to the parent TAlbum structure
  int quality;
  unsigned long landscape_width;
  unsigned long landscape_height;
  unsigned long portrait_width;
  unsigned long portrait_height;
} TAlbum;

typedef struct {
  char *path;
  char *title;
  char *title_description;
  char *thumbnail;
  char *preview;
  int quality;
  unsigned long width;
  unsigned long height;
  gboolean nofullsize;
} TIndexItem;



/*
 *   parse_album_xml: XML parser for gallery index.xml files  
 */ 
gboolean parse_album_xml (const char *filename, TAlbum *index);

/*
 *   free_album_data: free allocated album data
 */
void free_album_data (TAlbum *index);

/*
 *   get_album_objects_count: retrieve number of items in specified album  
 */ 
int get_album_objects_count (const char *filename);


#endif /* __ITEMS_H__ */
