/*   Cataract - Static web photo gallery generator
 *   Copyright (C) 2008 Tomas Bzatek <tbzatek@users.sourceforge.net>
 *   
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License
 *   as published by the Free Software Foundation; either version 2
 *   of the License, or any later version.
 *   
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *   
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
#include <glib.h>

#include <libxml/xmlmemory.h>
#include <libxml/xpath.h>


typedef struct {
	xmlDocPtr doc;
    xmlXPathContextPtr xpathCtx; 
} TXMLFile;



/*
 *   xml_parser_load: initialize and load the XML document    
 */ 
TXMLFile * xml_parser_load (const char *filename);

/*
 *   xml_parser_close: close the XML document parser    
 */ 
void xml_parser_close (TXMLFile *file);

/*
 *   xml_file_get_node_value: retrieve value from XPath node
 */
char * xml_file_get_node_value (TXMLFile *file, const char *x_path);

/*
 *   xml_file_get_node_attribute: retrieve attribute value from XPath node
 */
char * xml_file_get_node_attribute (TXMLFile *file, const char *x_path, const char *attr);
long xml_file_get_node_attribute_long (TXMLFile *file, const char *x_path, const char *attr, const int _default);

/*
 *   xml_file_get_node_present: existency test of the XPath node
 */
gboolean xml_file_get_node_present (TXMLFile *file, const char *x_path);

/*
 *   xml_file_node_get_children_count: retrieve number of children items of the specified XPath node
 */
int xml_file_node_get_children_count (TXMLFile *file, const char *x_path);
